unit UMainForm;

{$mode Delphi}{$H+}

interface

uses
  XMLConf, Classes, SysUtils, FileUtil, Forms, Controls, Graphics,
  Dialogs, ExtCtrls, ComCtrls, Menus, ActnList, UCore, UPlatform, Math,
  DateUtils, GraphType, UPersistentForm, UApplicationInfo, UCoolTranslator,
  LCLType, URegistry;

type

  { TMainForm }

  TMainForm = class(TForm)
    AAbout: TAction;
    ApplicationInfo1: TApplicationInfo;
    AShowRawImageDesc: TAction;
    AShowMap: TAction;
    ANewGame: TAction;
    AExit: TAction;
    AFullScreen: TAction;
    ActionList1: TActionList;
    CoolTranslator1: TCoolTranslator;
    Image1: TImage;
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    MenuItem8: TMenuItem;
    StatusBar1: TStatusBar;
    TimerDraw: TTimer;
    TimerEngineTick: TTimer;
    XMLConfig1: TXMLConfig;
    procedure AExitExecute(Sender: TObject);
    procedure AFullScreenExecute(Sender: TObject);
    procedure ANewGameExecute(Sender: TObject);
    procedure AShowMapExecute(Sender: TObject);
    procedure AShowRawImageDescExecute(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure Image1MouseLeave(Sender: TObject);
    procedure Image1Resize(Sender: TObject);
    procedure TimerDrawTimer(Sender: TObject);
    procedure TimerEngineTickTimer(Sender: TObject);
  private
    OriginalBounds: TRect;
    OriginalWindowState: TWindowState;
    ScreenBounds: TRect;
    StartTime: TDateTime;
    DrawDuration: TDateTime;
    Drawing: Boolean;
    procedure LoadConfig;
    procedure SaveConfig;
  public
    PersistentForm: TPersistentForm;
    Engine: TEngine;
    procedure EraseBackground(DC: HDC); override;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

uses
  UMapForm, UNewGameForm;

resourcestring
  SRound = '%0:s of %1:s';

{ TMainForm }

procedure TMainForm.TimerDrawTimer(Sender: TObject);
begin
  Application.ProcessMessages;
  if not Drawing then
  try
    Drawing := True;
    StartTime := NowPrecise;
    //Engine.Draw;
    try
      Engine.Lock.Acquire;
      //StatusBar1.Panels[1].Text := IntToStr(TPlayer(Engine.Players[0]).Position.X) + ', ' +
      //  IntToStr(TPlayer(Engine.Players[0]).Position.Y) + ' ' +
      //  IntToStr(TPlayer(Engine.Players[0]).Direction);
      StatusBar1.Panels[2].Text := FloatToStr(RoundTo(Engine.DrawDuration / OneMillisecond, -2));
      StatusBar1.Panels[3].Text := Format(SRound, [IntToStr(Engine.CurrentRound),
        IntToStr(Engine.MaxRound)]);
    finally
      Engine.Lock.Release;
    end;
  finally
    Drawing := False;
  end;
end;

procedure TMainForm.TimerEngineTickTimer(Sender: TObject);
begin
  //Engine.Tick;
end;

procedure TMainForm.LoadConfig;
begin
  if not FileExists(XMLConfig1.Filename) then Exit;
  if Assigned(Engine) then Engine.LoadConfig(XMLConfig1, '');
end;

procedure TMainForm.SaveConfig;
begin
  if Assigned(Engine) then Engine.SaveConfig(XMLConfig1, '');
  ForceDirectories(ExtractFileDir(XMLConfig1.Filename));
  XMLConfig1.Flush;
end;

procedure TMainForm.EraseBackground(DC: HDC);
begin
  // Do nothing
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  XMLConfig1.Filename := GetAppConfigDir(False) + 'Config.xml';

  PersistentForm := TPersistentForm.Create(nil);
  PersistentForm.RegistryContext := TRegistryContext.Create(ApplicationInfo1.RegistryRoot,
    ApplicationInfo1.RegistryKey);

  Application.OnDeactivate := FormDeactivate;
  Randomize;
  DoubleBuffered := True;
  Engine := TEngine.Create;
  Engine.Bitmap := Image1.Picture.Bitmap;
  LoadConfig;
  Engine.NewGame;
  Engine.Active := True;
  Image1Resize(Self);

  Image1.ControlStyle := Image1.ControlStyle + [csOpaque];
end;

procedure TMainForm.FormDeactivate(Sender: TObject);
begin
  Engine.KeyBoard.ClearAll;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  PersistentForm.Free;
  Engine.Free;
end;

procedure TMainForm.AFullScreenExecute(Sender: TObject);
begin
  if BorderStyle <> bsNone then begin
    // To full screen
    OriginalWindowState := WindowState;
    OriginalBounds := BoundsRect;

    BorderStyle := bsNone;
    ScreenBounds := Screen.MonitorFromWindow(Handle).BoundsRect;
    with ScreenBounds do
      SetBounds(Left, Top, Right - Left, Bottom - Top) ;
  end else begin
    // From full screen
    {$IFDEF MSWINDOWS}
    BorderStyle := bsSizeable;
    {$ENDIF}
    if OriginalWindowState = wsMaximized then
      WindowState := wsMaximized
    else
      with OriginalBounds do
        SetBounds(Left, Top, Right - Left, Bottom - Top) ;
    {$IFDEF LINUX}
    BorderStyle := bsSizeable;
    {$ENDIF}
  end;
end;

procedure TMainForm.ANewGameExecute(Sender: TObject);
begin
  NewGameForm.LoadData(Engine);
  if NewGameForm.ShowModal = mrOk then begin
    NewGameForm.SaveData(Engine);
    Engine.NewGame;
  end;
end;

procedure TMainForm.AShowMapExecute(Sender: TObject);
begin
  MapForm.Show;
end;

procedure TMainForm.AShowRawImageDescExecute(Sender: TObject);
begin
  ShowMessage(Image1.Picture.Bitmap.RawImage.Description.AsString);
end;

procedure TMainForm.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  PersistentForm.Save(Self);
  SaveConfig;
end;

procedure TMainForm.AExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  Engine.KeyBoard.KeyState[Key] := True;
  StatusBar1.Panels[0].Text := IntToStr(Key);
end;

procedure TMainForm.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState
  );
begin
  Engine.KeyBoard.KeyState[Key] := False;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  PersistentForm.Load(Self);
  CoolTranslator1.Language := CoolTranslator1.Languages.SearchByCode('cs');
end;

procedure TMainForm.Image1MouseLeave(Sender: TObject);
begin

end;

procedure TMainForm.Image1Resize(Sender: TObject);
begin
  try
    Engine.Lock.Acquire;
    Image1.Picture.Bitmap.SetSize(Image1.Width, Image1.Height);
    Engine.ResizePlayerFrames;
  finally
    Engine.Lock.Release;
  end;
end;

end.

